use axum::{http::StatusCode, routing::get, Router};
use tokio::net::TcpListener;
use tracing::Level;
use tracing_subscriber::FmtSubscriber;

use wrbapp_server::database;

#[tokio::main]
async fn main() {
    dotenvy::dotenv().ok();

    // Initialize logging
    let subscriber = FmtSubscriber::builder()
        .with_max_level(Level::INFO)
        .finish();

    tracing::subscriber::set_global_default(subscriber)
        .expect("Error while initialized tracing subscriber");

    // Initialize database connection
    database::apply_migrations()
        .await
        .expect("Database migrations failed");

    database::connect()
        .await
        .expect("Database connection failed");

    // Serve app
    let app = Router::new().route("/", get(hello_world));

    let listener = TcpListener::bind("127.0.0.1:3000")
        .await
        .expect("Error while initializing listener");

    tracing::info!("Listening on {}", listener.local_addr().unwrap());

    axum::serve(listener, app)
        .await
        .expect("Error while serving axum application");
}

async fn hello_world() -> Result<String, (StatusCode, String)> {
    Ok("Hello world".to_string())
}
